#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "ladspa.h"

#define DCREM_INPUT                    0
#define DCREM_OUTPUT                   1

LADSPA_Descriptor *dcRemDescriptor = NULL;

typedef struct {
	LADSPA_Data *input;
	LADSPA_Data *output;
	LADSPA_Data  itm1;
	LADSPA_Data  otm1;
	LADSPA_Data run_adding_gain;
} DcRem;

const LADSPA_Descriptor *ladspa_descriptor(unsigned long index) {
	switch (index) {
	case 0:
		return dcRemDescriptor;
	default:
		return NULL;
	}
}

void activateDcRem(LADSPA_Handle instance) {
	DcRem *plugin_data = (DcRem *)instance;
	LADSPA_Data itm1 = plugin_data->itm1;
	LADSPA_Data otm1 = plugin_data->otm1;
#line 16 "dc_remove_1207.xml"
	itm1 = 0.0f;
	otm1 = 0.0f;

}

void cleanupDcRem(LADSPA_Handle instance) {
	free(instance);
}

void connectPortDcRem(
 LADSPA_Handle instance,
 unsigned long port,
 LADSPA_Data *data) {
	DcRem *plugin;

	plugin = (DcRem *)instance;
	switch (port) {
	case DCREM_INPUT:
		plugin->input = data;
		break;
	case DCREM_OUTPUT:
		plugin->output = data;
		break;
	}
}

LADSPA_Handle instantiateDcRem(
 const LADSPA_Descriptor *descriptor,
 unsigned long s_rate) {
	DcRem *plugin_data = (DcRem *)malloc(sizeof(DcRem));
	plugin_data->run_adding_gain = 1.0f;

	return (LADSPA_Handle)plugin_data;
}

#undef buffer_write
#undef RUN_ADDING
#undef RUN_REPLACING

#define buffer_write(b, v) (b = v)
#define RUN_ADDING    0
#define RUN_REPLACING 1

void runDcRem(LADSPA_Handle instance, unsigned long sample_count) {
	DcRem *plugin_data = (DcRem *)instance;

	/* Input (array of floats of length sample_count) */
	LADSPA_Data *input = plugin_data->input;

	/* Output (array of floats of length sample_count) */
	LADSPA_Data *output = plugin_data->output;
	LADSPA_Data itm1 = plugin_data->itm1;
	LADSPA_Data otm1 = plugin_data->otm1;

#line 21 "dc_remove_1207.xml"
	unsigned long pos;

	for (pos = 0; pos < sample_count; pos++) {
	  otm1 = 0.999f * otm1 + input[pos] - itm1;
	  itm1 = input[pos];
	  buffer_write(output[pos], otm1);
	}

	plugin_data->itm1 = itm1;
	plugin_data->otm1 = otm1;
}
#undef buffer_write
#undef RUN_ADDING
#undef RUN_REPLACING

#define buffer_write(b, v) (b += (v) * run_adding_gain)
#define RUN_ADDING    1
#define RUN_REPLACING 0

void setRunAddingGainDcRem(LADSPA_Handle instance, LADSPA_Data gain) {
	((DcRem *)instance)->run_adding_gain = gain;
}

void runAddingDcRem(LADSPA_Handle instance, unsigned long sample_count) {
	DcRem *plugin_data = (DcRem *)instance;
	LADSPA_Data run_adding_gain = plugin_data->run_adding_gain;

	/* Input (array of floats of length sample_count) */
	LADSPA_Data *input = plugin_data->input;

	/* Output (array of floats of length sample_count) */
	LADSPA_Data *output = plugin_data->output;
	LADSPA_Data itm1 = plugin_data->itm1;
	LADSPA_Data otm1 = plugin_data->otm1;

#line 21 "dc_remove_1207.xml"
	unsigned long pos;

	for (pos = 0; pos < sample_count; pos++) {
	  otm1 = 0.999f * otm1 + input[pos] - itm1;
	  itm1 = input[pos];
	  buffer_write(output[pos], otm1);
	}

	plugin_data->itm1 = itm1;
	plugin_data->otm1 = otm1;
}

void _init() {
	char **port_names;
	LADSPA_PortDescriptor *port_descriptors;
	LADSPA_PortRangeHint *port_range_hints;

	dcRemDescriptor =
	 (LADSPA_Descriptor *)malloc(sizeof(LADSPA_Descriptor));

	if (dcRemDescriptor) {
		dcRemDescriptor->UniqueID = 1207;
		dcRemDescriptor->Label = strdup("dcRem");
		dcRemDescriptor->Properties =
		 LADSPA_PROPERTY_HARD_RT_CAPABLE;
		dcRemDescriptor->Name =
		 strdup("DC Offset Remover");
		dcRemDescriptor->Maker =
		 strdup("Steve Harris <steve@totl.net>");
		dcRemDescriptor->Copyright =
		 strdup("GPL");
		dcRemDescriptor->PortCount = 2;

		port_descriptors = (LADSPA_PortDescriptor *)calloc(2,
		 sizeof(LADSPA_PortDescriptor));
		dcRemDescriptor->PortDescriptors =
		 (const LADSPA_PortDescriptor *)port_descriptors;

		port_range_hints = (LADSPA_PortRangeHint *)calloc(2,
		 sizeof(LADSPA_PortRangeHint));
		dcRemDescriptor->PortRangeHints =
		 (const LADSPA_PortRangeHint *)port_range_hints;

		port_names = (char **)calloc(2, sizeof(char*));
		dcRemDescriptor->PortNames =
		 (const char **)port_names;

		/* Parameters for Input */
		port_descriptors[DCREM_INPUT] =
		 LADSPA_PORT_INPUT | LADSPA_PORT_AUDIO;
		port_names[DCREM_INPUT] =
		 strdup("Input");
		port_range_hints[DCREM_INPUT].HintDescriptor = 0;

		/* Parameters for Output */
		port_descriptors[DCREM_OUTPUT] =
		 LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO;
		port_names[DCREM_OUTPUT] =
		 strdup("Output");
		port_range_hints[DCREM_OUTPUT].HintDescriptor = 0;

		dcRemDescriptor->activate = activateDcRem;
		dcRemDescriptor->cleanup = cleanupDcRem;
		dcRemDescriptor->connect_port = connectPortDcRem;
		dcRemDescriptor->deactivate = NULL;
		dcRemDescriptor->instantiate = instantiateDcRem;
		dcRemDescriptor->run = runDcRem;
		dcRemDescriptor->run_adding = runAddingDcRem;
		dcRemDescriptor->set_run_adding_gain = setRunAddingGainDcRem;
	}
}

void _fini() {
	int i;

	if (dcRemDescriptor) {
		free((char *)dcRemDescriptor->Label);
		free((char *)dcRemDescriptor->Name);
		free((char *)dcRemDescriptor->Maker);
		free((char *)dcRemDescriptor->Copyright);
		free((LADSPA_PortDescriptor *)dcRemDescriptor->PortDescriptors);
		for (i = 0; i < dcRemDescriptor->PortCount; i++)
			free((char *)(dcRemDescriptor->PortNames[i]));
		free((char **)dcRemDescriptor->PortNames);
		free((LADSPA_PortRangeHint *)dcRemDescriptor->PortRangeHints);
		free(dcRemDescriptor);
	}

}
